{% extends 'base.html' %}
{% load i18n %}
{% block title %}{% trans "Application Catalog" %}{% endblock %}

{% block page_header %}
  {% include "horizon/common/_page_header.html" with title=_("Application Catalog") %}
{% endblock page_header %}

{% block main %}

<div ng-cloak
  ng-controller="appCatalogTableCtrl" ng-init='init({{ APP_CATALOG_SETTINGS }})'>
  <ng-include src="'{{ STATIC_URL }}dashboard/project/app_catalog/main_panel.html'"></ng-include>
</div>
{% endblock %}

